#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os

import xml.sax
from xml.sax.saxutils import XMLGenerator
from xml.sax import handler
from xml.sax.xmlreader import InputSource

from pybookreader.utils import *

##----------------------------------------------------------------------
##----------------------------------------------------------------------


class PyBRContentHandler(handler.ContentHandler):

    def __init__(self):

        self.curr_type = None
        self.curr_data = ""
        self.stack = []

        self.books_list = []
        self.categories_list = {}


    def startElement(self, name, attrs):

        self.curr_data = ''

        if name == 'category':

            self.stack.append(self.curr_type)
            self.curr_type = 'category'
            name = attrs['name']
            self.curr_cat = name
            self.categories_list[name] = []


        elif name == 'book':

            self.stack.append(self.curr_type)
            self.curr_type = 'book'
            self.curr_book = Book()
            self.books_list.append(self.curr_book)
            self.categories_list[self.curr_cat].append(self.curr_book)


        elif name == 'mark':

            self.stack.append(self.curr_type)
            self.curr_type = 'mark'
            self.curr_mark = _Mark()
            self.curr_mark.number = int(attrs['number'])
            self.curr_book.marks_list.append(self.curr_mark)
            

##----------------------------------------------------------------------

    def endElement(self, name):

        if name == 'mark':
            self.curr_type = self.stack.pop()

        elif name == 'book':
            self.curr_type = self.stack.pop()


        elif name == 'mark':
            self.curr_type = self.stack.pop()


        elif self.curr_type == 'mark':

            if name == 'offset': # todo: check (PyBRErrorHandler)
                self.curr_mark.offset = int(self.curr_data)

            elif name == 'position': # todo: check (PyBRErrorHandler)
                self.curr_mark.position = float(self.curr_data)

            elif name == 'description':
                self.curr_mark.description = self.curr_data

            elif name == 'author':
                self.curr_mark.author = self.curr_data

            elif name == 'name':
                self.curr_mark.name = self.curr_data

            else:
                print 'unknown tag in "mark" field: ', name

        elif self.curr_type == 'book':


            if name == 'filename':
                self.curr_book.filename = self.curr_data

            elif name == 'author':
                self.curr_book.author = self.curr_data

            elif name == 'name':
                self.curr_book.name = self.curr_data

            elif name == 'description':
                self.curr_book.description = self.curr_data

            elif name == 'position': # todo: check self.curr_data
                self.curr_book.current_position = float(self.curr_data)

            elif name == 'encoding':
                self.curr_book.encoding = self.curr_data

            else:
                print 'unknown tag in "book" field: ', name


##----------------------------------------------------------------------

    def characters(self, data):
        self.curr_data += data


##----------------------------------------------------------------------

#  ,      DTD-

class PyBRErrorHandler(handler.ErrorHandler): pass # TODO
class PyBREntityResolver(handler.EntityResolver): pass
class PyBRDTDHandler(handler.DTDHandler): pass


##----------------------------------------------------------------------
##----------------------------------------------------------------------


class Bookmarks:

    def __init__(self):

        self.categories_list = {}
        self.books_list = []

        if not os.path.exists(bookmarks_file): return

        #   :
        content_handler = PyBRContentHandler()
        error_handler = PyBRErrorHandler()
        entity_resolver = PyBREntityResolver()
        dtd_handler = PyBRDTDHandler()

        #   :
        input = open(bookmarks_file)
        input_source = InputSource()
        input_source.setByteStream(input)

        #  XML-:

        #   :
        xml_reader = xml.sax.make_parser()

        #  : 
        xml_reader.setContentHandler(content_handler)
        xml_reader.setErrorHandler(error_handler)
        xml_reader.setEntityResolver(entity_resolver)
        xml_reader.setDTDHandler(dtd_handler)

        #   XML:
        xml_reader.parse(input_source)

        self.books_list = content_handler.books_list
        self.categories_list = content_handler.categories_list

##----------------------------------------------------------------------

    def make_new_category(self, name):
        if not self.categories_list.has_key(name) and len(name) > 0:
            self.categories_list[name] = []

##----------------------------------------------------------------------

    def make_new_book(self, filename):

        book=Book()
        book.filename = filename
        self.books_list.append(book)

        self.make_new_category('Default')
        self.categories_list['Default'].append(book)

        return book

##----------------------------------------------------------------------

    def move_book(self, filename, new_cat):
        '''move book to new category'''

        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break

        if found:
            self.categories_list[new_cat].append(book)

        if found: return 1
        else: return 0


##----------------------------------------------------------------------

    def remove_book(self, filename):

        # remove from categories_list
        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break

        # remove from books_list
        if found:
            n = 0
            for b in self.books_list:
                if b.filename == filename:
                    del self.books_list[n]
                    break

                n = n+1

        return found


##----------------------------------------------------------------------

    def remove_bookmark(self, filename, mark_number):
        found = 0
        for b in self.books_list:
            if b.filename == filename:
                n = 0
                for m in b.marks_list:
                    if m.number == mark_number:
                        del b.marks_list[n]
                        found = 1
                        break
                    n = n+1
        return found

##----------------------------------------------------------------------

    def get_book(self, filename):
        for b in self.books_list:
            if b.filename == filename: return b
        return None

##----------------------------------------------------------------------

    def get_mark(self, book, mark_number):
        for b in book.marks_list:
            if b.number == mark_number:
                return b
        return None

##----------------------------------------------------------------------

    def save_bookmarks(self):

        file = open(bookmarks_file, 'w')

        g = XMLGenerator(file, 'utf-8')

        g.startDocument()

        g.startElement('pybookreader', {'version': program_version})

        for cat in self.categories_list:

            g.startElement('category', {'name': cat})

            for book in self.categories_list[cat]:

                g.startElement('book', {})

                g.startElement('filename',{})
                g.characters(book.filename)
                g.endElement('filename')

                g.startElement('author',{})
                g.characters(book.author)
                g.endElement('author')

                g.startElement('name',{})
                g.characters(book.name)
                g.endElement('name')

                g.startElement('description',{})
                g.characters(book.description)
                g.endElement('description')

                g.startElement('position',{})
                g.characters(str(book.current_position))
                g.endElement('position')

                g.startElement('encoding',{})
                g.characters(book.encoding)
                g.endElement('encoding')

                # marks
                for mark in book.marks_list:

                    g.startElement('mark', {'number': str(mark.number)})

                    g.startElement('description', {})
                    g.characters(mark.description)
                    g.endElement('description')

                    g.startElement('offset', {})
                    g.characters(str(mark.offset))
                    g.endElement('offset')

                    g.startElement('position', {})
                    g.characters(str(mark.position))
                    g.endElement('position')

                    g.startElement('author',{})
                    g.characters(mark.author)
                    g.endElement('author')

                    g.startElement('name',{})
                    g.characters(mark.name)
                    g.endElement('name')

                    g.endElement('mark')

                g.endElement('book')

            g.endElement('category')

        g.endElement('pybookreader')

        g.endDocument()

        file.write('\n')
        file.close()


##----------------------------------------------------------------------

class Book:

    def __init__(self):

        self.category = 'Default'
        self.filename = None # abs path
        self.author = ''
        self.name = ''
        self.description = ''
        self.current_position = 0.0
        self.encoding = 'koi8-r' # FIXME
        self.data_type = 'plain_text'
        self.marks_list = []

    def add_bookmark(self, position, offset, description):

        mark = _Mark()
        mark.position = position
        mark.offset = offset
        mark.description = description

        # search uniq number
        num = 0
        while 1:
            found = 0
            for i in self.marks_list:
                if i.number == num:
                    found = 1
                    break
            if not found:
                break
            num = num+1

        mark.number = num
        self.marks_list.append(mark)

        # sort by position
        self.marks_list.sort(lambda x, y: cmp(x.position, y.position))


##----------------------------------------------------------------------

class Category:

    def __init__(self):

        self.name = 'Default'
        self.description = ''

##----------------------------------------------------------------------

class _Mark:

    def __init__(self):

        self.number = 0
        self.position = 0.0
        self.description = ''

        self.author = ''
        self.name = ''

        self.offset = 0


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    bookmarks = Bookmarks()

    print '****** books_list ******'
    print

    for book in bookmarks.books_list:

        print 'category =', book.category
        print 'filename =', book.filename
        print 'author =', book.author
        print 'name =', book.name
        print 'description =', book.description
        print 'current_position =', book.current_position
        print 'encoding =', book.encoding
        print

        print '   *** marks_list ***'
        print

        for mark in book.marks_list:

            print '   namber =', mark.number
            print '   offset =', mark.offset
            print '   position =', mark.position
            print '   description =', mark.description
            print '   author =', mark.author
            print '   name =', mark.name
            print

