#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os

import gtk, gobject
from gtk import TRUE, FALSE

from pybookreader.bookmarks import *
from pybookreader.utils import *


##----------------------------------------------------------------------
##----------------------------------------------------------------------


class BookmarksWindow(gtk.Window):

    width = 600
    height = 400


    COLUMN_FOLDER     = 0
    COLUMN_FILENAME   = 1
    COLUMN_AUTHOR     = 2
    COLUMN_NAME       = 3
    COLUMN_PERCENT    = 4
    COLUMN_DECRIPTION = 5
    COLUMN_PATH       = 6 # invisible
    COLUMN_MARK       = 7 # invisible


    def __init__(self, bookmarks):


        gtk.Window.__init__(self)

        self.preview_length = 1000 # todo: to properties?

        self.books_list = bookmarks.books_list
        self.categories_list = bookmarks.categories_list
        self.bookmarks = bookmarks

        self.set_title('%s: Bookmarks manager' % program_name)

        self.connect('destroy', self.destroy_cb)
        self.connect('size_allocate', self.size_allocate_cb)


        top_vbox =  gtk.VBox()
        self.add(top_vbox)


        bbox = gtk.HButtonBox()
        top_vbox.pack_start(bbox, expand=FALSE)


        bbox.set_border_width(0)
        bbox.set_layout(gtk.BUTTONBOX_START)
        bbox.set_spacing(0)
        #bbox.set_property('homogeneous', FALSE) # don't work
        #bbox.set_homogeneous(TRUE)

        button = gtk.Button('_Add file')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.add_file_cb)


        button = gtk.Button('_New category')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.add_category_cb)

        button = gtk.Button('_Delete')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.delete_clicked_cb)

        button = gtk.Button('A_pply')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.apply_clicked_cb)

        button = gtk.Button('_Load')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.ok_clicked_cb)

        button = gtk.Button('_Close')
        button.set_property('relief', gtk.RELIEF_NONE)
        bbox.add(button)
        button.connect('clicked', self.close_clicked_cb)


        self.paned = gtk.VPaned()
        top_vbox.add(self.paned)



        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        #scrolled_window.set_shadow_type(gtk.SHADOW_IN)

        self.paned.pack1(scrolled_window)


        # ...
        treeview = gtk.TreeView()
        treeview.set_rules_hint(TRUE)
        #treeview.set_search_column(self.COLUMN_FILENAME) # ???

        scrolled_window.add(treeview)


        # category
        column = gtk.TreeViewColumn('Category', gtk.CellRendererText(),
                                    text=self.COLUMN_FOLDER)
        treeview.append_column(column)
        # filename
        column = gtk.TreeViewColumn('File', gtk.CellRendererText(),
                                    text=self.COLUMN_FILENAME)
        treeview.append_column(column)
        # author
        column = gtk.TreeViewColumn('Author', gtk.CellRendererText(),
                                    text=self.COLUMN_AUTHOR)
        treeview.append_column(column)
        # name
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(),
                                    text=self.COLUMN_NAME)
        treeview.append_column(column)
        # percent
        column = gtk.TreeViewColumn('%', gtk.CellRendererText(),
                                    text=self.COLUMN_PERCENT)
        treeview.append_column(column)
        # description
        column = gtk.TreeViewColumn('Description', gtk.CellRendererText(),
                                    text=self.COLUMN_DECRIPTION)
        treeview.append_column(column)

        treeview.connect('row-activated', self.treeview_row_activated_cb)

        selection = treeview.get_selection()
        selection.connect('changed', self.treeview_selected_cb)


        self.treeview = treeview

        vbox=gtk.VBox()
        self.paned.pack2(vbox)


        table = gtk.Table(5, 2)
        vbox.pack_start(table, expand=FALSE, fill=TRUE, padding=0)

        label = gtk.Label('File: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        table.attach(label, 0, 1, 0, 1, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.file_label = gtk.Label('')
        table.attach(self.file_label, 1, 4, 0, 1, xpadding=4, ypadding=4)

        label = gtk.Label('Ca_tegory: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 0, 1, 1, 2, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.category_menu = gtk.OptionMenu()
        table.attach(self.category_menu, 1, 2, 1, 2, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.category_menu)

        label = gtk.Label('_Encoding: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 2, 3, 1, 2, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.encoding_menu = gtk.OptionMenu()
        table.attach(self.encoding_menu, 3, 4, 1, 2, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.encoding_menu)

        label = gtk.Label('A_uthor: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 0, 1, 2, 3, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.author_entry = gtk.Entry()
        table.attach(self.author_entry, 1, 2, 2, 3, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.author_entry)

        label = gtk.Label('Na_me: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 2, 3, 2, 3, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.name_entry = gtk.Entry()
        table.attach(self.name_entry, 3, 4, 2, 3, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.name_entry)

        label = gtk.Label('De_scription: ')
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.set_use_underline(TRUE)
        table.attach(label, 0, 1, 3, 4, xoptions=gtk.FILL,
                     xpadding=4, ypadding=4)
        self.description_entry = gtk.Entry()
        table.attach(self.description_entry, 1, 4, 3, 4, xpadding=4, ypadding=4)
        label.set_mnemonic_widget(self.description_entry)


        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        vbox.pack_end(scrolled_window)

        self.text_view = gtk.TextView()
        scrolled_window.add(self.text_view)

        self.text_view.set_editable(FALSE)
        self.text_view.set_cursor_visible(FALSE)
        self.text_view.set_wrap_mode(gtk.WRAP_WORD)


        self.encoding_submenu = gtk.Menu()
        for i in charsets_list:
            item = gtk.MenuItem(i)
            self.encoding_submenu.append(item)
            item.connect_object('activate', self.encodind_submenu_activate_cb, i)
            item.show()
        self.encoding_menu.set_menu(self.encoding_submenu)


        self.category_submenu = gtk.Menu()
        if len(self.categories_list) == 0: # add 'Default' item
            i = 'Default'
            item = gtk.MenuItem(i)
            self.category_submenu.append(item)
            item.connect_object('activate',
                                self.category_submenu_activate_cb, i)
            item.show()
        else:
            for i in self.categories_list:
                item = gtk.MenuItem(i)
                self.category_submenu.append(item)
                item.connect_object('activate',
                                    self.category_submenu_activate_cb, i)
                item.show()
        self.category_menu.set_menu(self.category_submenu)


        self.update_treeview()

##----------------------------------------------------------------------

    def encodind_submenu_activate_cb(self, enc):

        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)

        books = self.books_list

        for i in books:
            if i.filename == filename:

                i.encoding = enc

                # FIXME: change description?

                self.treeview_selected_cb(selection)

                break

##----------------------------------------------------------------------

    def category_submenu_activate_cb(self, new_cat):

        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)
        if not filename: return

        found = self.bookmarks.move_book(filename, new_cat)

        self.update_treeview()

##         # treeview update
##         if found:
##             # serch iter of new category
##             new_iter = model.get_iter_first()
##             while 1:
##                 val = model.get_value(new_iter, self.COLUMN_FOLDER)
##                 if val == new_cat:
##                     break
##                 new_iter = model.iter_next(new_iter)
##                 if not new_iter:
##                     break
##             if not new_iter: return
##             model.remove(iter)
##             # add new row
##             book_iter = model.append(new_iter)
##             model.set(book_iter,
##                       self.COLUMN_FILENAME, os.path.basename(book.filename),
##                       self.COLUMN_NAME, book.name,
##                       self.COLUMN_AUTHOR, book.author,
##                       self.COLUMN_PERCENT, int(book.current_position*100),
##                       self.COLUMN_DECRIPTION, book.description,
##                       self.COLUMN_PATH, book.filename)
##             self.treeview.expand_row(model.get_path(new_iter) ,TRUE)


##----------------------------------------------------------------------

    def treeview_selected_cb(self, sel):

        model, iter = sel.get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        books = self.books_list


        found = 0
        for b in books:
            if b.filename == filename:

                self.file_label.set_text(b.filename)


                if mark:
                    for m in b.marks_list:
                        if str(m.number) == mark:

                            if m.author:
                                self.author_entry.set_text(m.author)
                            else:
                                self.author_entry.set_text(b.author)
                            if m.name:
                                self.name_entry.set_text(m.name)
                            else:
                                self.name_entry.set_text(b.name)
                            self.description_entry.set_text(m.description)

                            # update text preview
                            text = open_file(filename,
                                             length=self.preview_length,
                                             offset=m.offset,
                                             encoding=b.encoding)

                else:

                    self.author_entry.set_text(b.author)
                    self.name_entry.set_text(b.name)
                    self.description_entry.set_text(b.description)

                    # update text preview
                    text = open_file(filename,
                                     length=self.preview_length,
                                     encoding=b.encoding)


                buffer = self.text_view.get_buffer()
                buffer.set_text(text)

                # update encoding menu
                n = 0
                for enc in charsets_list:
                    if b.encoding == enc:
                        break
                    n = n+1
                # bug ?
                self.encoding_menu.remove_menu()
                self.encoding_submenu.set_active(n)
                self.encoding_menu.set_menu(self.encoding_submenu)

                # update category menu
                n = 0
                for cat in self.categories_list:
                    if b in self.categories_list[cat]:
                        break
                    n = n+1

                self.category_menu.remove_menu()
                self.category_submenu.set_active(n)
                self.category_menu.set_menu(self.category_submenu)


                found = 1
                break

        if not found:
            self.clear_view()

##----------------------------------------------------------------------

    def destroy_cb(self, w):
        pass

##----------------------------------------------------------------------

    def size_allocate_cb(self, w, rect):
        self.width = rect.width
        self.height = rect.height

##----------------------------------------------------------------------

    def add_file_cb(self, w):

        dialog = OpenFileDialog(self)
        filename = dialog.filename

        if filename:

            book = self.bookmarks.make_new_book(filename)

            data = open_file(filename,
                             length=default_description_length,
                             encoding=default_encoding)
            desc = create_description(data)

            book.description = desc
            book.encoding = default_encoding

            self.update_treeview()


##----------------------------------------------------------------------

    def add_category_cb(self, w, a=None):

        selection = self.treeview.get_selection()
        model, iter = selection.get_selected()

        dialog = gtk.Dialog('dialog', self,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_OK, gtk.RESPONSE_OK,
                             gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))

        hbox = gtk.HBox()
        dialog.vbox.pack_start(hbox, TRUE, TRUE, 0)

        label = gtk.Label('Category: ')
        hbox.pack_start(label, expand=FALSE)

        entry = gtk.Entry()

        hbox.pack_start(entry)

        entry.connect('activate', lambda w:
                      dialog.emit ('response', gtk.RESPONSE_OK))

        dialog.show_all()

        response = dialog.run()

        if response == gtk.RESPONSE_OK:

            name = entry.get_text()
            if self.bookmarks.categories_list.has_key(name):
                error_dialog(dialog, 'Name must be unique')
            elif len(name) == 0:
                pass
            else:

                cat_iter = self.model.append(None)
                self.model.set(cat_iter, self.COLUMN_FOLDER, name)
                self.bookmarks.make_new_category(name)

                item = gtk.MenuItem(name)
                self.category_submenu.append(item)
                item.connect_object('activate',
                                    self.category_submenu_activate_cb,
                                    name)
                item.show()

        dialog.destroy()

##----------------------------------------------------------------------

    def apply_clicked_cb(self, w):

        model, iter = self.treeview.get_selection().get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        books = self.books_list

        for b in books:
            if b.filename == filename:

                if mark:
                    for m in b.marks_list:
                        if str(m.number) == mark:

                            m.author = self.author_entry.get_text()
                            m.name = self.name_entry.get_text()
                            m.description = self.description_entry.get_text()

                            model.set(iter,
                                      self.COLUMN_NAME, m.name,
                                      self.COLUMN_AUTHOR, m.author,
                                      self.COLUMN_DECRIPTION, m.description)
                            break

                else:

                    b.author = self.author_entry.get_text()
                    b.name = self.name_entry.get_text()
                    b.description = self.description_entry.get_text()

                    model.set(iter,
                              self.COLUMN_NAME, b.name,
                              self.COLUMN_AUTHOR, b.author,
                              self.COLUMN_DECRIPTION, b.description)

                break

##----------------------------------------------------------------------

    def treeview_row_activated_cb(self, w, row, col): # double click
        self.emit_load_file()

##----------------------------------------------------------------------

    def ok_clicked_cb(self, w):
        self.emit_load_file()

##----------------------------------------------------------------------

    def emit_load_file(self):

        model, iter = self.treeview.get_selection().get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        if mark:
            b = self.bookmarks.get_book(filename)
            if b:
                m = self.bookmarks.get_mark(b, int(mark))
                if m:
                    b.current_position = m.position
                    self.emit('load-file', filename)

        elif filename:
            self.emit('load-file', filename)


##----------------------------------------------------------------------

    def delete_clicked_cb(self, w):

        model, iter = self.treeview.get_selection().get_selected()
        if not iter: return

        filename = model.get_value(iter, self.COLUMN_PATH)
        mark = model.get_value(iter, self.COLUMN_MARK)

        if mark: # mark

            if self.bookmarks.remove_bookmark(filename, int(mark)):
                self.clear_view()
                model.remove(iter)

        elif filename: # book

            if self.bookmarks.remove_book(filename):
                self.clear_view()
                model.remove(iter)
                self.emit('book-removed', filename)

        else: # category

            cat = model.get_value(iter, self.COLUMN_FOLDER)

            if model.iter_has_child(iter):
                error_dialog(self, 'Category must be empty')
            else:
                del self.bookmarks.categories_list[cat]
                self.update_treeview()

##----------------------------------------------------------------------

    def clear_view(self):

        self.file_label.set_text('')
        self.author_entry.set_text('')
        self.name_entry.set_text('')
        self.description_entry.set_text('')
        buffer = self.text_view.get_buffer()
        buffer.set_text('')


##----------------------------------------------------------------------

    def close_clicked_cb(self, w):
        self.destroy()

##----------------------------------------------------------------------

    def update_treeview(self):

        model = gtk.TreeStore(gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING)

        self.model = model


        if len(self.categories_list) > 0:
            for cat in self.categories_list: # categories
                iter = model.append(None)
                model.set(iter, self.COLUMN_FOLDER, cat)

                for book in self.categories_list[cat]: # books
                    book_iter = model.append(iter)

                    model.set(book_iter,
                              self.COLUMN_FILENAME, os.path.basename(book.filename),
                              self.COLUMN_NAME, book.name,
                              self.COLUMN_AUTHOR, book.author,
                              self.COLUMN_PERCENT, str(int(book.current_position*100)),
                              self.COLUMN_DECRIPTION, book.description,
                              self.COLUMN_PATH, book.filename)

                    for mark in book.marks_list: # marks
                        mark_iter = model.append(book_iter)

                        model.set(mark_iter,
                                  self.COLUMN_NAME, mark.name,
                                  self.COLUMN_AUTHOR, mark.author,
                                  self.COLUMN_PERCENT, str(int(mark.position*100)),
                                  self.COLUMN_DECRIPTION, mark.description,
                                  self.COLUMN_PATH, book.filename,
                                  self.COLUMN_MARK, mark.number)

        self.treeview.set_model(model)

        self.treeview.expand_all()

##----------------------------------------------------------------------

gobject.signal_new('load-file', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

gobject.signal_new('book-removed', BookmarksWindow,
                   gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))

##----------------------------------------------------------------------

if(__name__=='__main__'):
    bookmarks = Bookmarks()
    bw = BookmarksWindow(bookmarks)
    bw.set_default_size(600, 600)
    bw.paned.set_position(300)
    bw.connect('destroy', lambda w: gtk.main_quit())
    bw.show_all()
    gtk.main()

