#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os, codecs

import gtk, pango
from gtk import TRUE, FALSE

from pybookreader.bookmarkswindow import *
from pybookreader.bookmarks import *
from pybookreader.utils import *
from pybookreader.scrolledtext import *



##----------------------------------------------------------------------
##----------------------------------------------------------------------

class PyBookReader(gtk.Window):


    def __init__(self):
        gtk.Window.__init__(self)

##----------------------------------------------------------------------

        # widgets
        self.scrolled_window = None
        self.text_widget = None
        self.text_buffer = None
        self.scale = None
        self.pause_toggle = None
        self.percent_status = None
        self.bookmarks_window = None


        self.percent_font = 'Sans Bold 16'

        self.main_properties = {
            # <name><type><value>
            'width'                          : ['int',   600      ],
            'height'                         : ['int',   400      ],
            'bookmarks_window_width'         : ['int',   600      ],
            'bookmarks_window_height'        : ['int',   400      ],
            'bookmarks_window_paned_position': ['int',   200      ],
            'font'                           : ['str',   'Normal' ],
            'fg_color'                       : ['str',   '#000000'],
            'bg_color'                       : ['str',   '#ffffff'],
            'speed'                          : ['float', 20.0     ],
            'filename'                       : ['str',   None     ],
            'file_select_path'               : ['str',   OpenFileDialog.file_select_path]
            }


        # block charset menu (fix it?)
        self.block_change_charset = 0

        self.bookmarks = None
        self.current_book = None

##----------------------------------------------------------------------

        self.bookmarks = Bookmarks()


        self.set_title(program_name)

        menu_items = [
            ('/_File', None, None, 0, '<Branch>' ),
            #('/_File/tearoff1', None, self.ifactory_cb, 0, '<Tearoff>'),
            ('/_File/_Open...', '<control>O', self.menu_open_cb, 0, ''),
            ('/_File/sep1', None, None, 0, '<Separator>'),
            ('/_File/_Quit', '<control>Q',
             lambda s, w: self.destroy(), 0, ''),
            ('/_File/Quit without saving', None,
             lambda s, w: gtk.main_quit(), 0, ''),

            ('/_Bookmarks', None, None, 0, '<Branch>' ),
            ('/_Bookmarks/Book_marks manager ...',
             '<control>B', self.menu_bookmarks_cb, 0, ''),
            ('/_Bookmarks/_Set bookmark',
             '<control>S', self.menu_set_bookmark_cb, 0, ''),
            ('/_Bookmarks/Se_t bookmark with ...',
             '<control>D', self.menu_set_bookmark_with_cb, 0, ''),
            ('/_Bookmarks/sep1', None, None, 0, '<Separator>'),
            ('/_Bookmarks/Go to _position',
             '<control>G', self.menu_goto_position_cb, 0, ''),
            #('/_Bookmarks/Go to _begin', None, self.menu_to_begin_cb, 0, ''),
            ('/_Bookmarks/Go to _begin', None, lambda s, w:
             self.text_widget.set_position(0.0), 0, ''),
            ('/_Bookmarks/Go to _end', None, lambda s, w:
             self.text_widget.set_position(1.0), 0, ''),

            ('/_Preferences', None, None, 0, '<Branch>'),
            ('/_Preferences/Fon_t...', None, self.menu_font_cb, 0, ''),
            ('/_Preferences/_Foreground color...',
             None, self.menu_color_cb, 0, ''),
            ('/_Preferences/_Background color...',
             None, self.menu_color_cb, 1, ''),
            #('/_Preferences/_Debug', None, self.menu_debug_cb, 2, ''),
            ('/_Preferences/sep1', None, None, 0, '<Separator>'),
            ('/_Preferences/_Properties...',
             None, self.menu_props_cb, 0, ''),

            ('/_Speed', None,  None, 0, '<Branch>'),
            ('/_Speed/_Fast  \t+',  None, self.menu_speed_cb, 0, ''),
            ('/_Speed/_Slow  \t-',  None, self.menu_speed_cb, 1, ''),
            ('/_Speed/_Pause \tInsert', None, self.menu_speed_cb, 2, ''),

            ('/_Charset', None, None, 0, '<Branch>'),
            ]

        path = '/Charset/' + charsets_list[0]
        menu_items.append((path, None, self.menu_charset_cb, 0, '<RadioItem>'))
        for i in range (1, len(charsets_list)):
            menu_items.append(('/Charset/' + charsets_list[i],
                               None, self.menu_charset_cb, i, path))

        accelgroup = gtk.AccelGroup()
        self.add_accel_group(accelgroup)
        self.accelgroup = accelgroup

        self.connect('destroy', self.destroy_cb)
        self.connect('size_allocate', self.size_allocate_cb)


        vbox = gtk.VBox()
        self.add(vbox)


        item_factory = gtk.ItemFactory(gtk.MenuBar, '<main>', accelgroup)
        item_factory.create_items(menu_items)
        self.item_factory = item_factory

        menubar = item_factory.get_widget('<main>')
        vbox.pack_start(menubar, expand=FALSE)


        self.text_widget = ScrolledText()
        vbox.pack_start(self.text_widget, expand=TRUE)

        self.text_widget.vadjustment.connect('value_changed',
                                             self.position_changed_cb)

        self.text_widget.connect('pause-toggled', self.pause_toggled)
        self.text_widget.connect('speed-changed', self.speed_changed)

        self.read_config()

        statusbar = self.create_statusbar()
        vbox.pack_end(statusbar, expand=FALSE)

        self.set_default_size(self.main_properties['width'][1],
                              self.main_properties['height'][1])

        self.show_all()

        filename = self.main_properties['filename'][1]
        if filename:
            try:
                self.insert_file(self.main_properties['filename'][1])
            except:
                pass


##----------------------------------------------------------------------

    # DEBUG
    def menu_debug_cb(self, a, w):
        print "debug"


        bookmarks = self.bookmarks

        print '****** books_list ******'
        print

        for book in bookmarks.books_list:

            print 'category =', book.category
            print 'filename =', book.filename
            print 'author =', book.author
            print 'name =', book.name
            print 'description =', string_encode(book.description,
                                                 'utf-8', 'koi8-r')
            print 'current_position =', book.current_position
            print 'encoding =', book.encoding
            print

            print '   *** marks_list ***'
            print

            for mark in book.marks_list:

                print '   namber =', mark.number
                print '   offset =', mark.offset
                print '   position =', mark.position
                print '   description =', string_encode(mark.description,
                                                        'utf-8', 'koi8-r')
                print '   author =', mark.author
                print '   name =', mark.name
                print



##----------------------------------------------------------------------

    def pause_toggled(self, w, a):
        self.pause_toggle.set_active(a)

##----------------------------------------------------------------------

    def speed_changed(self, w, a):
        adj = self.scale.get_adjustment()
        adj.set_value (a)

##----------------------------------------------------------------------

    def destroy_cb(self, w):

        try: # debug
            self.save_bookmark()
            self.bookmarks.save_bookmarks()
            self.save_config()
        finally:
            gtk.main_quit()

##----------------------------------------------------------------------

    def size_allocate_cb(self, w, rect):
        self.main_properties['width'][1] = rect.width
        self.main_properties['height'][1] = rect.height

##----------------------------------------------------------------------

    def position_changed_cb(self, w):

        self.bookmark_value = self.text_widget.get_position()
        percent = int(self.bookmark_value*100)
        self.percent_status.set_text('%d%%' % percent)

##----------------------------------------------------------------------

    def speed_changed_cb(self, w):

        adj = self.scale.get_adjustment()
        self.text_widget.speed = adj.get_value()
        self.text_widget.speed_changed()

##----------------------------------------------------------------------

    def menu_open_cb(self, a, w):

        dialog = OpenFileDialog(self)
        if dialog.filename:
            self.insert_file(dialog.filename, 1)

##----------------------------------------------------------------------

    def menu_bookmarks_cb(self, a, w):
        if not self.bookmarks_window:
            self.bookmarks_window = BookmarksWindow(self.bookmarks)

            self.bookmarks_window.connect('destroy',
                                          self.bookmarks_window_destroy_cb)
            self.bookmarks_window.connect('load-file',
                                          self.bookmarks_window_load_file_cb)
            self.bookmarks_window.connect('book-removed',
                                          self.bookmarks_window_book_removed_cb)

            width = self.main_properties['bookmarks_window_width'][1]
            height = self.main_properties['bookmarks_window_height'][1]
            self.bookmarks_window.set_default_size(width, height)
            pos = self.main_properties['bookmarks_window_paned_position'][1]
            self.bookmarks_window.paned.set_position(pos)
            self.bookmarks_window.show_all()

        else:
            #gtk.gdk.Window.raise(self.bookmarks_window) # ???
            pass

##----------------------------------------------------------------------

    def bookmarks_window_destroy_cb(self, w):
        self.main_properties['bookmarks_window_width'][1] = w.width
        self.main_properties['bookmarks_window_height'][1] = w.height
        self.main_properties['bookmarks_window_paned_position'][1] \
             = w.paned.get_position()
        self.bookmarks_window = None

##----------------------------------------------------------------------

    def bookmarks_window_load_file_cb(self, w, filename):

        if self.main_properties['filename'][1] != filename:
            self.insert_file(filename)
        else:
            self.text_widget.set_position(self.current_book.current_position)

##----------------------------------------------------------------------

    def bookmarks_window_book_removed_cb(self, w, filename):

        if self.main_properties['filename'][1] == filename:
            if len(self.bookmarks.books_list) != 0:
                self.insert_file(self.bookmarks.books_list[0].filename)
            else:
                self.set_title(program_name)
                # clear buffer
                start, end = self.text_widget.text_buffer.get_bounds()
                self.text_widget.text_buffer.delete(start, end)

                self.main_properties['filename'][1] = None
                self.current_book = None


##----------------------------------------------------------------------

    def menu_goto_position_cb(self, a, w):

        dialog = gtk.Dialog('dialog', self,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_OK, gtk.RESPONSE_OK,
                             gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        hbox = gtk.HBox()
        dialog.vbox.pack_start(hbox, TRUE, TRUE, 0)
        label = gtk.Label('Go to (%) : ')
        hbox.pack_start(label, expand=FALSE)
        entry = gtk.Entry()
        entry.connect('activate', lambda w:
                      dialog.emit ('response', gtk.RESPONSE_OK))
        hbox.pack_start(entry)
        dialog.show_all()
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            strng = entry.get_text()
            try:
                num = float(strng)
            except:
                pass
            else:
                if num >= 0 and num <= 100:
                    self.text_widget.set_position(num/100.0)
        dialog.destroy()

##----------------------------------------------------------------------

    # FONT
    def menu_font_cb(self, a, w):

        dialog = gtk.FontSelectionDialog("%s: Select font" % program_name)
        fontname = self.text_widget.get_style().font_desc.to_string()
        dialog.set_font_name(fontname)
        dialog.set_transient_for(self)
        dialog.show()

        response = dialog.run()

        if response == gtk.RESPONSE_OK:
            font = dialog.get_font_name()
            fd = pango.FontDescription(font)
            self.text_widget.modify_font(fd)

        dialog.destroy()

##----------------------------------------------------------------------

    # COLOR
    def menu_color_cb(self, a, w):

        dialog = gtk.ColorSelectionDialog("%s: Color Select" % program_name)
        dialog.set_transient_for(self)
        dialog.colorsel.set_has_palette(TRUE)
        dialog.show()

        response = dialog.run()

        if response == gtk.RESPONSE_OK:

            color = dialog.colorsel.get_current_color()
            if a == 0: # foreground
                self.text_widget.fg_color = '#%02x%02x%02x' \
                                            % (color.red>>8,
                                               color.green>>8,
                                               color.blue>>8)
                self.text_widget.modify_text(gtk.STATE_NORMAL, color) # text fg
            else:      # background
                self.text_widget.bg_color = '#%02x%02x%02x' \
                                            % (color.red>>8,
                                               color.green>>8,
                                               color.blue>>8)
                self.text_widget.modify_base(gtk.STATE_NORMAL, color) # text bg

        dialog.destroy()

##----------------------------------------------------------------------

    def menu_props_cb(self, a, w):
        dialog = gtk.Dialog('dialog', self,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_OK, gtk.RESPONSE_OK,
                             gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))

        notebook = gtk.Notebook()
        dialog.vbox.pack_start(notebook, expand=TRUE, fill=TRUE)


        table = gtk.Table(len(self.text_widget.scroll_properties), 2)
        table.set_row_spacings(4)
        table.set_col_spacings(4)

        vbox = gtk.VBox()
        vbox.pack_start(table, expand=FALSE, fill=FALSE)

        label = gtk.Label('')
        label.set_text_with_mnemonic('_Scrolling')
        notebook.append_page(vbox, label)

        for i in self.text_widget.scroll_properties:

            n = self.text_widget.scroll_properties[i][2]
            label = gtk.Label(self.text_widget.scroll_properties[i][1])
            label.set_use_underline(TRUE)
            table.attach_defaults(label, 0, 1, n, n+1)
            spin = gtk.SpinButton()
            spin.set_range(self.text_widget.scroll_properties[i][3],
                           self.text_widget.scroll_properties[i][4])
            spin.set_increments(1, 4)
            spin.set_value(self.text_widget.scroll_properties[i][0])
            spin.connect('value_changed', self.spin_value_changed_2, i)
            table.attach_defaults(spin, 1, 2, n, n+1)
            label.set_mnemonic_widget(spin)

        for i in self.text_widget.scroll_type:
            n = self.text_widget.scroll_type[i][2]
            check = gtk.CheckButton(self.text_widget.scroll_type[i][1])
            check.set_active(self.text_widget.scroll_type[i][0])
            check.connect('toggled', self.check_toggled, i)
            table.attach_defaults(check, 0, 2, n+3, n+4)

        table = gtk.Table(len(self.text_widget.text_properties), 2)
        table.set_row_spacings(4)
        table.set_col_spacings(4)

        vbox = gtk.VBox()
        vbox.pack_start(table, expand=FALSE, fill=FALSE)

        label = gtk.Label('')
        label.set_text_with_mnemonic('_Text')
        notebook.append_page(vbox, label)


        for i in self.text_widget.text_properties:
            n = self.text_widget.text_properties[i][2]
            label = gtk.Label(self.text_widget.text_properties[i][1])
            label.set_use_underline(TRUE)
            label.set_justify(gtk.JUSTIFY_LEFT)
            table.attach_defaults(label, 0, 1, n, n+1)
            spin = gtk.SpinButton()
            spin.set_range(self.text_widget.text_properties[i][3],
                           self.text_widget.text_properties[i][4])
            spin.set_increments(1, 4)
            spin.set_value(self.text_widget.text_properties[i][0])
            spin.connect('value_changed', self.spin_value_changed, i)
            table.attach_defaults(spin, 1, 2, n, n+1)
            #table.attach(spin, 1, 2, n, n+1, gtk.EXPAND | gtk.FILL,
            #             gtk.EXPAND | gtk.FILL, 2, 2)
            #table.child_set_property(spin, 'x_options', gtk.EXPAND | gtk.FILL)
            label.set_mnemonic_widget(spin)

        self.scroll_properties_tmp = {}
        self.scroll_type_tmp = {}
        self.text_properties_tmp = {}

        dialog.show_all()

        response = dialog.run()

        if response == gtk.RESPONSE_OK:

            if len(self.scroll_properties_tmp) != 0:
                for i in self.scroll_properties_tmp:
                    self.text_widget.scroll_properties[i][0] \
                         = self.scroll_properties_tmp[i]

            if len(self.scroll_type_tmp) != 0:
                for i in self.scroll_type_tmp:
                    self.text_widget.scroll_type[i][0] \
                         = self.scroll_type_tmp[i]

            if len(self.text_properties_tmp) != 0:
                for i in self.text_properties_tmp:
                    self.text_widget.text_properties[i][0] \
                         = self.text_properties_tmp[i]

                self.text_widget.set_properties()

        del self.scroll_properties_tmp
        del self.scroll_type_tmp
        del self.text_properties_tmp

        dialog.destroy()

##----------------------------------------------------------------------

    def check_toggled(self, w, name):
        self.scroll_type_tmp[name] = w.get_active()

##----------------------------------------------------------------------

    def spin_value_changed(self, w, name):
        self.text_properties_tmp[name] = w.get_value_as_int()

##----------------------------------------------------------------------

    def spin_value_changed_2(self, w, name):
        self.scroll_properties_tmp[name] = w.get_value_as_int()

##----------------------------------------------------------------------

    def menu_set_bookmark_cb(self, a, w):

        rect = self.text_widget.get_visible_rect()
        iter = self.text_widget.get_iter_at_location(rect.x, rect.y)
        offset = iter.get_offset()
        iter_start = iter.copy()
        iter.forward_chars(default_description_length)
        text = self.text_widget.text_buffer.get_text(iter_start, iter, TRUE)
        desc = create_description(text)
        self.current_book.add_bookmark(self.text_widget.get_position(),
                                       offset, desc)
        if self.bookmarks_window:
            self.bookmarks_window.update_treeview()

##----------------------------------------------------------------------

    def menu_set_bookmark_with_cb(self, a, w):

        dialog = gtk.Dialog('dialog', self,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_OK, gtk.RESPONSE_OK,
                             gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        hbox = gtk.HBox()
        dialog.vbox.pack_start(hbox, TRUE, TRUE, 0)
        label = gtk.Label('Desription: ')
        hbox.pack_start(label, expand=FALSE)
        entry = gtk.Entry()
        entry.connect('activate', lambda w:
                      dialog.emit ('response', gtk.RESPONSE_OK))
        hbox.pack_start(entry)
        dialog.show_all()
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            rect = self.text_widget.get_visible_rect()
            iter = self.text_widget.get_iter_at_location(rect.x, rect.y)
            offset = iter.get_offset()
            desc = entry.get_text()
            self.current_book.add_bookmark(self.text_widget.get_position(),
                                           offset, desc)
            if self.bookmarks_window:
                self.bookmarks_window.update_treeview()

        dialog.destroy()

##----------------------------------------------------------------------

    def get_desc_from_text(self):
        # not used

        rect = self.text_widget.get_visible_rect()
        iter = self.text_widget.get_iter_at_location(rect.x, rect.y)
        iter_start = iter.copy()
        iter.forward_chars(default_description_length)
        text = self.text_widget.text_buffer.get_text(iter_start, iter, TRUE)
        desc = create_description(text)

        return desc

##----------------------------------------------------------------------

    def menu_speed_cb(self, a, w):
        if a == 0  : # fast
            adj = self.scale.get_adjustment()
            adj.set_value(adj.value + adj.step_increment)
        elif a == 1: # slow
            adj = self.scale.get_adjustment()
            adj.set_value(adj.value - adj.step_increment)
        elif a == 2: # pause
            self.pause_toggle.set_active(not self.pause_toggle.get_active())

##----------------------------------------------------------------------

    def menu_charset_cb(self, a, w):
        # FIXME: change description?

        if not self.current_book: return
        if self.current_book.encoding == charsets_list[a]: return
        if self.block_change_charset: return

        self.current_book.encoding = charsets_list[a]
        self.insert_file(self.current_book.filename)

##----------------------------------------------------------------------

    def pause_toggled_cb(self, w):
        self.text_widget.pause_toggled(w.get_active())

##----------------------------------------------------------------------

    def create_statusbar(self):

        hbox = gtk.HBox()

        label_speed = gtk.Label('Speed: ')
        hbox.pack_start(label_speed, expand=FALSE, padding=4)

        scale = gtk.HScale()
        scale.set_size_request(200, -1)
        scale.set_range(1, 100)
        scale.set_increments(1, 1)
        scale.set_value(self.text_widget.speed)
        scale.set_property('draw_value', FALSE)
        scale.set_property('can_focus', FALSE)
        scale.connect('value_changed', self.speed_changed_cb)
        self.scale = scale

        hbox.pack_start(scale, expand=TRUE, padding=4)


        toggle = gtk.ToggleButton()
        self.pause_toggle = toggle
        toggle.set_active(TRUE)
        #toggle.set_use_underline(TRUE)
        toggle.set_label('Pause')
        toggle.set_property('can_focus', FALSE)
        toggle.set_property('border_width', 4)
        toggle.connect('toggled', self.pause_toggled_cb)
        toggle.set_accel_path('<main>/Speed/Pause', self.accelgroup)

        hbox.pack_start(toggle, expand=FALSE)

        self.statusbar_1 = gtk.Label('')
        hbox.pack_start(self.statusbar_1, expand=TRUE)
        #self.statusbar_1.connect('button_release_event',
        #                         self.percent_button_press_cb)

        self.percent_status = gtk.Label('0%')
        hbox.pack_start(self.percent_status, fill=TRUE, expand=FALSE)
        #self.percent_status.connect('button_press_event',
        #                     self.percent_status_button_press_cb)

        self.percent_status.modify_font(pango.FontDescription(self.percent_font))

        grid = gtk.Statusbar()
        hbox.pack_start(grid, expand=FALSE)
        grid.set_size_request(24, 24)

        return hbox

##----------------------------------------------------------------------

    def percent_button_press_cb(self, w, event):
        print event

##----------------------------------------------------------------------

    def insert_file(self, filename, update_treeview=0):

        # --- check file ---

        if not os.path.isfile(filename):
            raise OpenFileError, 'is not an existing regular file'

        try:
            file =  open(filename)
        except IOError, err:
            raise OpenFileError, err
        else:
            file.close()


        # --- search book ---

        current_book = None

        book_found = 0

        for i in self.bookmarks.books_list:
            if i.filename == filename:
                current_book = i
                book_found = 1
                break

        if self.current_book:
            self.save_bookmark()


        # make new 'Book' object

        if not book_found:
            current_book = self.bookmarks.make_new_book(filename)
            current_book.encoding = default_encoding
        else:
            pass

        self.current_book = current_book

        self.main_properties['filename'][1] = filename #???


        # --- load file ---

        self.set_title('%s: %s' % (program_name, os.path.basename(filename)))

        data = open_file(filename, encoding=current_book.encoding)

        # create description
        if not book_found:
            desc = create_description(data) # FIXME: to utf-8?
            current_book.description = desc

        if self.bookmarks_window and update_treeview:
            self.bookmarks_window.update_treeview()


        # clear buffer
        start, end = self.text_widget.text_buffer.get_bounds()
        self.text_widget.text_buffer.delete(start, end)
        # insert data
        iter = self.text_widget.text_buffer.get_iter_at_offset(0)
        self.text_widget.text_buffer.insert(iter, data)

        # set position
        if book_found: gtk.idle_add(self.set_position)

##----------------------------------------------------------------------

    def set_position(self):

        self.text_widget.set_position(self.current_book.current_position)

        self.block_change_charset = 1
        menu_item = self.item_factory.get_item('/Charset/' \
                                               + self.current_book.encoding)
        menu_item.set_active(TRUE)
        self.block_change_charset = 0

##----------------------------------------------------------------------

    def save_bookmark(self):

        #if not self.main_properties['filename'][1]: return
        if not self.current_book: return

        self.current_book.current_position = self.text_widget.get_position()


##----------------------------------------------------------------------

    def save_config(self):

        self.main_properties['font'][1] \
            = self.text_widget.get_style().font_desc.to_string()


        color = self.text_widget.get_style().text[gtk.STATE_NORMAL] # text fg
        fg = '#%02x%02x%02x' % (color.red>>8, color.green>>8, color.blue>>8)
        self.main_properties['fg_color'][1] = fg


        color = self.text_widget.get_style().base[gtk.STATE_NORMAL] # text bg
        bg = '#%02x%02x%02x' % (color.red>>8, color.green>>8, color.blue>>8)
        self.main_properties['bg_color'][1] = bg


        self.main_properties['speed'][1] = self.text_widget.speed
        self.main_properties['file_select_path'][1] = OpenFileDialog.file_select_path


        # todo: mkdir

        try:
            file = open(conf_file, 'w')
        except:
            print "WARNING: can't open configuration file:", conf_file
            return

        file.write('[pybookreader]\n')

        for i in self.main_properties:
            if i == 'filename' and self.main_properties[i][1] == None:
                # FIXME: this is hack: if filename == None
                pass
            else:
                file.write('%s=%s\n' % (i, self.main_properties[i][1]))


        for i in self.text_widget.scroll_properties:
            file.write('%s=%s\n' % (i, self.text_widget.scroll_properties[i][0]))

        for i in self.text_widget.scroll_type:
            file.write('%s=%s\n' % (i, self.text_widget.scroll_type[i][0]))

        for i in self.text_widget.text_properties:
            file.write('%s=%s\n' % (i, self.text_widget.text_properties[i][0]))


        file.write('\n')

        file.close()


##----------------------------------------------------------------------

    def read_config(self):

        try:
            file=open(conf_file)
        except:
            return

        properties = {}
        flag = 0

        for line in file.readlines():
            if line == '[pybookreader]\n':
                flag = 1
                continue
            elif line[0] == '[':
                flag = 0
            elif flag == 1:
                indx = string.find(line, '=')
                if indx == -1:
                    continue
                properties[line[0:indx]] = line[indx+1:len(line)-1]


        for i in self.main_properties:
            if properties.has_key(i):
                if self.main_properties[i][0] == 'int':
                    self.main_properties[i][1] = int(properties[i])
                elif self.main_properties[i][0] == 'float':
                    self.main_properties[i][1] = float(properties[i])
                else:
                    self.main_properties[i][1] = properties[i]


        font = pango.FontDescription(self.main_properties['font'][1])
        self.text_widget.modify_font(font)

        color = gtk.gdk.color_parse(self.main_properties['fg_color'][1])
        self.text_widget.modify_text(gtk.STATE_NORMAL, color)

        color = gtk.gdk.color_parse(self.main_properties['bg_color'][1])
        self.text_widget.modify_base(gtk.STATE_NORMAL, color)

        self.text_widget.speed = self.main_properties['speed'][1]
        self.text_widget.speed_changed()

        OpenFileDialog.file_select_path = self.main_properties['file_select_path'][1]

        for i in self.text_widget.text_properties:
            if properties.has_key(i):
                self.text_widget.text_properties[i][0] = int(properties[i])

        for i in self.text_widget.scroll_properties:
            if properties.has_key(i):
                self.text_widget.scroll_properties[i][0] = int(properties[i])

        for i in self.text_widget.scroll_type:
            if properties.has_key(i):
                self.text_widget.scroll_type[i][0] = int(properties[i])

        self.text_widget.set_properties()


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    br = PyBookReader()
    gtk.main()

