#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os, exceptions, codecs, gzip
import gtk


program_name = 'PyBookReader'
program_version = '0.4.1'

conf_file = os.path.expanduser('~/.pybr_conf')
bookmarks_file = os.path.expanduser('~/.pybr_bookmarks')

charsets_list = ('koi8-r', 'cp1251', 'cp866', 'iso8859-5', 'utf-8')

# FIXME
default_description_length = 200
# FIXME: set system encoding or add to properties???
default_encoding = 'koi8-r'

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class OpenFileError(exceptions.Exception):
    """OpenFileError exception"""
    def __init__(self, args=None):
        self.args = args


##----------------------------------------------------------------------

def open_file(filename, length=None, offset=None, encoding='utf-8'):

    if filename[len(filename)-3:len(filename)] == '.gz': # gzipped
        file = gzip.open(filename)
    elif filename[len(filename)-4:len(filename)] in ('.zip', '.rar'):
        raise OpenFileError, 'file iz zip/rar archive, not supported yet'
    else:
        file = open(filename)

    if offset:
        file.seek(offset) # FIXME: if file is utf-8

    if length:
        data = file.read(length)
    else:
        data = file.read()

    file.close()

    # delete '\0' chars
    tbl = string.maketrans('', '')
    data = string.translate(data, tbl, '\0')

    if encoding != 'utf-8':
        from_enc = encoding
        to_enc   = 'utf-8'

        inenc,  indec,  sr, sw = codecs.lookup(from_enc)
        outenc, outdec, sr, sw = codecs.lookup(to_enc)

        data = outenc(indec(data)[0])[0]

    return data


##----------------------------------------------------------------------

def string_encode(data, from_enc, to_enc='utf-8'):

        inenc,  indec,  sr, sw = codecs.lookup(from_enc)
        outenc, outdec, sr, sw = codecs.lookup(to_enc)
        return outenc(indec(data)[0])[0]

##----------------------------------------------------------------------

def error_dialog(parent, msg):

    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL \
                               | gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                               msg)

    dialog.run()
    dialog.destroy()

##----------------------------------------------------------------------

def create_description(data):
    desc = ''
    for i in data:
        if string.find(string.whitespace, i) >= 0 or i == "'":
            if len(desc) != 0 and desc[len(desc)-1] != ' ':
                desc = desc + ' '
        else:
            desc = desc + i

        if len(desc) > default_description_length \
           and desc[len(desc)-1] == ' ': break

    desc = desc + '...' # ???

    return desc

##----------------------------------------------------------------------

class OpenFileDialog:

    file_select_path = os.path.expanduser('~/')

    def __init__(self, win):

        dialog = gtk.FileSelection()
        dialog.set_transient_for(win)
        dialog.set_filename(self.file_select_path)
        dialog.show()
        self.filename = None

        while 1:

            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                filename = dialog.get_filename()

                try:
                    file = open (filename)
                except IOError, err:
                    error_dialog(dialog, 'Cannot open file: %s' % err)
                else:
                    file.close()
                    self.filename = filename
                    OpenFileDialog.file_select_path = os.path.dirname(filename) + os.sep
                    dialog.destroy()
                    return
            else:
                dialog.destroy()
                return

##----------------------------------------------------------------------
